<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Api\Data;

use Magento\Framework\Api\CustomAttributesDataInterface;

/**
 * Interface MultiWidgetInterface
 */
interface MultiWidgetInterface extends CustomAttributesDataInterface
{
    const TABLE                 = 'flat101_multiwidget';
    const ID                    = 'entity_id';
    const NAME                  = 'name';
    const NAME_HEADING          = 'name_heading';
    const DESCRIPTION           = 'description';
    const DESCRIPTION_HEADING   = 'description_heading';
    const STATUS                = 'status';
    const VISIBLE_FROM          = 'visible_from';
    const VISIBLE_TO            = 'visible_to';
    const TYPE_ID               = 'type_id';

    /**
     * Retrieve name
     *
     * @return string
     */
    public function getName(): string;

    /**
     * Set name
     *
     * @param string $name
     */
    public function setName(string $name);

    /**
     * Retrieve name heading
     *
     * @return string
     */
    public function getNameHeading(): ?string;

    /**
     * Set name heading
     *
     * @param string $nameHeading
     */
    public function setNameHeading(string $nameHeading);

    /**
     * Retrieve description
     *
     * @return string
     */
    public function getDescription(): string;

    /**
     * Set description
     *
     * @param string $description
     */
    public function setDescription(string $description);

    /**
     * Retrieve description heading
     *
     * @return string
     */
    public function getDescriptionHeading(): ?string;

    /**
     * Set description heading
     *
     * @param string $descriptionHeading
     */
    public function setDescriptionHeading(string $descriptionHeading);

    /**
     * Retrieve status
     *
     * @return int|null
     */
    public function getStatus(): ?int;

    /**
     * Set status
     *
     * @param int $status
     */
    public function setStatus(int $status);

    /**
     * Retrieve visible from date
     *
     * @return string|null
     */
    public function getVisibleFrom(): ?string;

    /**
     * Set visible from date
     *
     * @param string $visibleFrom
     */
    public function setVisibleFrom($visibleFrom);

    /**
     * Retrieve visible to date
     *
     * @return string|null
     */
    public function getVisibleTo(): ?string;

    /**
     * Set visible to date
     *
     * @param string $visibleTo
     */
    public function setVisibleTo($visibleTo);

    /**
     * Retrieve type id
     *
     * @return int|null
     */
    public function getTypeId(): ?int;

    /**
     * Set product type id
     *
     * @param int $typeId
     */
    public function setTypeId(int $typeId);

    /**
     * Retrieve existing extension attributes object or create a new one.
     *
     * @return \Flat101\MultiWidget\Api\Data\MultiWidgetExtensionInterface|null
     */
    public function getExtensionAttributes();

    /**
     * Set an extension attributes object.
     *
     * @param \Flat101\MultiWidget\Api\Data\MultiWidgetExtensionInterface $extensionAttributes
     */
    public function setExtensionAttributes(\Flat101\MultiWidget\Api\Data\MultiWidgetExtensionInterface $extensionAttributes);
}
