<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Api;

use Flat101\MultiWidget\Api\Data\ItemsSearchResultsInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetSearchResultsInterface;
use Flat101\MultiWidget\Model\ResourceModel\Items\Collection as ItemsCollection;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotDeleteException;

interface MultiWidgetRepositoryInterface
{
    /**
     * Save MultiWidget
     *
     * @param MultiWidgetInterface $multiWidget
     * @return MultiWidgetInterface
     */
    public function save(MultiWidgetInterface $multiWidget): MultiWidgetInterface;

    /**
     * Retrieve MultiWidget by id
     *
     * @param int $entityId
     * @return MultiWidgetInterface
     * @throws NoSuchEntityException
     */
    public function getById(int $entityId): MultiWidgetInterface;

    /**
     * Retrieve MultiWidget by attribute
     *
     * @param $value
     * @param string|null $attributeCode
     * @return MultiWidgetInterface
     * @throws NoSuchEntityException
     */
    public function get($value, ?string $attributeCode): MultiWidgetInterface;

    /**
     * Delete $multiWidget.
     *
     * @param MultiWidgetInterface $multiWidget
     * @return boolean
     * @throws CouldNotDeleteException
     */
    public function delete(MultiWidgetInterface $multiWidget): bool;

    /**
     * Delete entity by ID.
     *
     * @param int $entityId
     * @return boolean
     * @throws NoSuchEntityException
     * @throws CouldNotDeleteException
     */
    public function deleteById(int $entityId): bool;

    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return MultiWidgetSearchResultsInterface
     */
    public function getList(SearchCriteriaInterface $searchCriteria): MultiWidgetSearchResultsInterface;

    /**
     * Get MultiWidget's Items
     *
     * @param MultiWidgetInterface $multiWidget
     * @param bool $onlyEnabled
     * @return ItemsSearchResultsInterface
     */
    public function getItems(MultiWidgetInterface $multiWidget, bool $onlyEnabled = false): ItemsSearchResultsInterface;

    /**
     * Get MultiWidget by Item id
     *
     * @param int $entityId
     * @param bool $onlyEnabled
     * @return ItemsSearchResultsInterface
     * @throws NoSuchEntityException
     */
    public function getItemsById(int $entityId, bool $onlyEnabled = false): ItemsSearchResultsInterface;
}
