<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Block\Adminhtml;

use Flat101\MultiWidget\Api\MultiWidgetRepositoryInterface;
use Magento\Backend\Block\Widget\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\UiComponent\Control\ButtonProviderInterface;

class DeleteButton extends GenericButton implements ButtonProviderInterface
{
    /** @var Context */
    protected Context $context;

    /** @var MultiWidgetRepositoryInterface */
    protected MultiWidgetRepositoryInterface $multiWidgetRepository;

    /**
     * @param Context $context
     * @param MultiWidgetRepositoryInterface $multiWidgetRepository
     */
    public function __construct(
        Context $context,
        MultiWidgetRepositoryInterface $multiWidgetRepository
    ) {
        parent::__construct($context);
        $this->context = $context;
        $this->multiWidgetRepository = $multiWidgetRepository;
    }

    /**
     * Return CMS block ID
     *
     * @return int|null
     */
    public function getMultiWidgetId(): ?int
    {
        try {
            return intval($this->multiWidgetRepository->getById(
                intval($this->context->getRequest()->getParam('entity_id'))
            )->getId());
        } catch (NoSuchEntityException $e) {
        }
        return null;
    }

    public function getButtonData(): array
    {
        $data = [];
        if ($this->getMultiWidgetId()) {
            $data = [
                'label' => __('Delete'),
                'class' => 'delete',
                'on_click' => 'deleteConfirm(\'' . __(
                        'Are you sure you want to do this?'
                    ) . '\', \'' . $this->getDeleteUrl() . '\', {"data": {}})',
                'sort_order' => 50,
            ];
        }
        return $data;
    }

    /**
     * URL to send delete requests to.
     *
     * @return string
     */
    public function getDeleteUrl(): string
    {
        return $this->getUrl('*/*/delete', ['id' => $this->getMultiWidgetId()]);
    }
}
