<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Controller\Adminhtml\Sliders;

use Flat101\MultiWidget\Api\MultiWidgetRepositoryInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Model\ResourceModel\MultiWidget\CollectionFactory;
use Flat101\MultiWidget\Helper\FileEraser;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Ui\Component\MassAction\Filter;
use Psr\Log\LoggerInterface;

class MassDelete extends Action implements HttpPostActionInterface
{
    /** @var Filter */
    private $filter;

    /** @var CollectionFactory */
    private $collectionFactory;

    /** @var MultiWidgetRepositoryInterface */
    private $multiWidgetRepository;

    /** @var LoggerInterface */
    private $logger;

    /** @var FileEraser */
    private $fileEraser;

    public function __construct(
        Context $context,
        Filter $filter,
        CollectionFactory $collectionFactory,
        MultiWidgetRepositoryInterface $multiWidgetRepository,
        FileEraser $fileEraser,
        LoggerInterface $logger
    ) {
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->multiWidgetRepository = $multiWidgetRepository;
        $this->fileEraser = $fileEraser;
        $this->logger = $logger;
        parent::__construct($context);
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        $collection = $this->filter->getCollection($this->collectionFactory->create());
        $multiWidgetDeleted = 0;
        $multiWidgetDeletedError = 0;
        /** @var MultiWidgetInterface $multiWidget */
        foreach ($collection->getItems() as $multiWidget) {
            try {
                $this->fileEraser->removeMultiWidgetImages($multiWidget, Upload::SLIDERS_IMAGE_PATH_DESTINATION);
                $this->multiWidgetRepository->delete($multiWidget);
                $multiWidgetDeleted++;
            } catch (LocalizedException $exception) {
                $this->logger->error($exception->getLogMessage());
                $multiWidgetDeletedError++;
            } catch (\Exception $e) {
                $this->logger->error($e->getMessage());
            }
        }

        if ($multiWidgetDeleted) {
            $this->messageManager->addSuccessMessage(
                __('A total of %1 record(s) have been deleted.', $multiWidgetDeleted)
            );
        }

        if ($multiWidgetDeletedError) {
            $this->messageManager->addErrorMessage(
                __(
                    'A total of %1 record(s) haven\'t been deleted. Please see server logs for more details.',
                    $multiWidgetDeletedError
                )
            );
        }

        return $this->resultFactory->create(ResultFactory::TYPE_REDIRECT)->setPath('*/*/index');
    }
}
