<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Model\Source;

use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Model\ResourceModel\MultiWidget\Collection;
use Flat101\MultiWidget\Model\ResourceModel\MultiWidget\CollectionFactory;
use Magento\Framework\Data\OptionSourceInterface;
use PhpParser\Node\Expr\AssignOp\Mul;

abstract class MultiWidget implements OptionSourceInterface
{
    /** @var CollectionFactory */
    protected CollectionFactory $collectionFactory;

    /** @var array */
    private array $options = [];

    /**
     * MultiWidget constructor.
     * @param CollectionFactory $collectionFactory
     */
    public function __construct(
        CollectionFactory $collectionFactory
    ) {
        $this->collectionFactory = $collectionFactory;
    }

    public function toOptionArray(): array
    {
        array_push($this->options, [
            'label' => __('Choose an option'),
            'value' => null,
        ]);

        foreach ($this->getCollection() as $item) {
            array_push($this->options, [
                'label' => $item->getData(MultiWidgetInterface::NAME),
                'value' => $item->getData(MultiWidgetInterface::ID),
            ]);
        }

        return $this->options;
    }

    /**
     * Get options in "key-value" format
     *
     * @return array
     */
    public function toArray(): array
    {
        return $this->getCollection()->toArray();
    }

    /**
     * @return Collection
     */
    abstract protected function  getCollection(): Collection;
}
