# Multi Widget M2 Module by Flat 101

## Starting 🚀

Multi Widget is a module that tries to standardize several features that we think almost each
all projects have like Key Benefits and Sliders.

This module could be more completed, and we will try to add new features/entities to the base module.

#### BASE ENTITIES

- Key Benefits
- Sliders

## Installation

Use the package manager Composer to install module from the Flat 101's private package repository.

First, you need to add the Flat 101's private package repository to the project. Once the 
repository has been added:  

```bash
composer require flat101/multi-widget
```

```bash
bin/magento setup:upgrade
bin/magento setup:di:compile
bin/magento cache:clean
```

## Add new entities

We can add new entities with purpose to complete the module or by a requirement from new project.
It will be very simple and quickly. You can see other entities to replicate folders and files and 
adjust with fields you need for the new entity. To add a new entity we need to follow this steps:

#### DATABASE

- Add to database table 'flat101_multiwidget_type' a new row. We only need a name. Take in mind 
  that ID number is very important in the next steps. It is mandatory to add a Data Patch for stable
  new entities.

#### ADMIN PANEL

- Create a new menu item. Edit file Flat101/MultiWidget/etc/adminhtml/menu.xml
- Create new controller's folder in Flat101/MultiWidget/Controller. Exists folder AbstractController
  for help you. See other controller folder to see actions. Review folder structure.
- Create layouts for this controllers in Flat101/MultiWidget/view/adminhtml/layout.
- Create UI Components for listing and forms in Flat101/MultiWidget/view/adminhtml/ui_component.
  Here, you need ID number from 'flat101_multiwidget_type' to filter results in listing.
- Add virtual type Image Uploader to current upload controller in Flat101/MultiWidget/etc/di.xml.
- Create DataProvider for UI component in Flat101/MultiWidget/Model/MultiWidget.
- Create UI Component actions for listing in Flat101/MultiWidget/UI/Component/Listing/Columns.
- Create button functionalities in Flat101/MultiWidget/Block/adminhtml. Review folder structure.
- Create new widget form for entity in Flat101/MultiWidget/etc/widget.xml.

#### FRONTEND

- Add templates for widget in Flat101/MultiWidget/view/frontend/widget. Review folder structure.

## Usage

Navigate to the menu corresponding to the entity. There you will find the necessary options for CRUD.

## Allow new file types

Follow this steps to add new allowed file type to image uploader. 

*NOTE: SVG file extension was removed by Magento for security reasons. Maybe not a good idea to add it.*

- Flat101/MultiWidget/etc/di.xml. Add to argument 'allowedExtensions' and 'allowedMimeTypes' corresponding
values.
```bash
  <virtualType name="Flat101\MultiWidget\Model\ImageUploader" type="Magento\Catalog\Model\ImageUploader">
    <arguments>
      <argument name="baseTmpPath" xsi:type="string">multi_widget/tmp/images</argument>
      <argument name="basePath" xsi:type="string">multi_widget</argument>
      <argument name="allowedExtensions" xsi:type="array">
        <item name="jpg" xsi:type="string">jpg</item>
        <item name="jpeg" xsi:type="string">jpeg</item>
        <item name="gif" xsi:type="string">gif</item>
        <item name="png" xsi:type="string">png</item>
        <item name="webp" xsi:type="string">webp</item>
        ... (add here new allowed extension)
      </argument>
        <argument name="allowedMimeTypes" xsi:type="array">
        <item name="jpg" xsi:type="string">image/jpg</item>
        <item name="jpeg" xsi:type="string">image/jpeg</item>
        <item name="gif" xsi:type="string">image/gif</item>
        <item name="png" xsi:type="string">image/png</item>
        <item name="webp" xsi:type="string">image/webp</item>
        ... (add here new allowed mime type)
      </argument>
    </arguments>
  </virtualType>
```

- Add to each needed form, the new file type to '<allowedExtensions>' inside image uploader configuration
image field (for each one).
```bash 
  <field name="image" formElement="imageUploader">
    <settings>
      <validation>
        <rule name="required-entry" xsi:type="boolean">true</rule>
      </validation>
      <label translate="true">Desktop Image (larger size)</label>
      <componentType>imageUploader</componentType>
    </settings>
    <formElements>
      <imageUploader>
        <settings>
          <allowedExtensions>jpg jpeg gif png webp ...(add here new allowed extension)</allowedExtensions>
          <maxFileSize>2097152</maxFileSize>
          <previewTmpl>Magento_Catalog/image-preview</previewTmpl>
          <openDialogTitle>Sliders Gallery</openDialogTitle>
          <initialMediaGalleryOpenSubpath>multi_widget/sliders</initialMediaGalleryOpenSubpath>
          <uploaderConfig>
            <param xsi:type="string" name="url">*/*/upload</param>
          </uploaderConfig>
        </settings>
      </imageUploader>
    </formElements>
  </field>
```

## Contributions
Pull requests are welcome. For major changes, please open an issue first to discuss what you would 
like to change.

## Authors

* **Juan Antonio Licer** - Flat 101 Team
